# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib import admin

from .models import Cache

# Register your models here.


class CacheAdmin(admin.ModelAdmin):
    list_display = ('type', 'key')
    list_filter = ('type', )
    search_fields = ('key', 'type', )


admin.site.register(Cache, CacheAdmin)



import pytest
from ..cache import lrucache
from ..models import Cache


@lrucache('{0}+{1}')
def sum(a, b):
    return a + b


@pytest.mark.django_db
def test_cache():
    a = 1
    b = 2
    c = sum(a, b)

    assert c == 3

    try:
        Cache.objects.get(key="1+2")
    except:
        pytest.fail("Cache not created")



from functools import wraps
from itertools import izip

from .cache import lrucache
from .models import Cache

checkpoint = lrucache


def checklist(key_str):
    def _lru_cache(func):
        @wraps(func)
        def wrapper(alist, *args, **kwargs):
            keys = [key_str.format(k, *args, **kwargs) for k in alist]
            type = '{}.{}'.format(func.__module__, func.__name__)

            item_key = dict(izip(alist, keys))
            key_value = {}

            for key in keys:
                try:
                    key_value[key] = Cache.objects.get(
                        key=key, type=type).results
                except Cache.DoesNotExist:
                    pass

            items_not_checked = [
                k for k in item_key if item_key[k] not in key_value]

            if items_not_checked:
                results = func(items_not_checked, *args, **kwargs)

                for item, result in izip(items_not_checked, results):
                    key = item_key[item]
                    Cache.objects.update_or_create(
                        key=key,
                        type=type,
                        defaults={
                            "results": result
                        }
                    )
                    key_value[item_key[item]] = result

            return [key_value[item_key[item]] for item in alist]

        return wrapper

    return _lru_cache


def uncheck(type=None, key=None):
    queryset = Cache.objects
    if type:
        queryset = queryset.filter(type=type)
    if key:
        queryset = queryset.filter(key=key)

    return queryset.delete()



import hashlib
import mimetypes
import os
import shutil
import tempfile
import urllib
import urlparse
from contextlib import contextmanager
from functools import wraps
from os.path import basename, exists, join

import requests
from slugify import slugify


@contextmanager
def tmp(tmpdir=u"./tmp"):
    if not exists(tmpdir):
        os.makedirs(tmpdir)

    path = tempfile.mkdtemp(dir=u"./tmp")
    try:
        yield path
    finally:
        if exists(path):
            shutil.rmtree(path)


def cache(outpath):
    def x(func):
        @wraps(func)
        def inner(*args, **kwargs):
            path = outpath.format(*args, **kwargs)
            mode = 'folder' if path.endswith('/') else 'file'

            if mode == "folder":
                opath = u'./tmp/%s/' % path.split('/')[-2]
            else:
                opath = u'./tmp/%s' % basename(path)

            if exists(opath):
                return opath

            with tmp() as tmpfolder:
                tmppath = join(tmpfolder, basename(opath))
                if mode == "folder" and not exists(tmppath):
                    os.makedirs(tmppath)

                final_path = func(*args, opath=tmppath, **kwargs)

                if final_path == tmppath:
                    shutil.move(final_path, opath)
                    return opath

                return final_path

        return inner

    return x


@cache("{0}")
def _local(name, url, opath):
    urllib.urlretrieve(url, opath)
    return opath


def is_url(url):
    return url.startswith("http")


def _get_filename_ext_from_response(r):
    if "ETag" in r.headers:
        name = slugify(r.headers["ETag"])
    else:
        name = hashlib.md5(r.url).hexdigest()

    if "Content-Type" in r.headers:
        ext = mimetypes.guess_extension(r.headers["Content-Type"])
    else:
        path = urlparse.urlparse(r.url).path
        ext = os.path.splitext(path)[1]

    return name, ext


def local(url):
    if os.path.exists(url):
        return url

    assert is_url(url), u"{} is not a url".format(url)

    r = requests.head(url)
    assert r.status_code == 200

    filename = "%s%s" % _get_filename_ext_from_response(r)
    return _local(filename, url)


def remote(filepath):
    assert os.path.exists(filepath), "file not exists"

    files = {
        'file': (os.path.basename(filepath), open(filepath, 'rb')),
    }

    resp = requests.post('https://file.io/', files=files).json()
    assert resp['success'], resp

    return resp['link']



# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2017-11-09 07:07
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djworkspace', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='cache',
            name='key',
            field=models.CharField(max_length=100),
        ),
        migrations.AlterField(
            model_name='cache',
            name='type',
            field=models.CharField(max_length=100),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models
from picklefield.fields import PickledObjectField

# Create your models here.


class Cache(models.Model):
    type = models.CharField(max_length=100)
    key = models.CharField(max_length=100)

    results = PickledObjectField(null=True)

    class Meta:
        unique_together = (('type', 'key'), )



import os
from os.path import dirname, join

import requests

from workspace import cache, local, remote, tmp

test_file = join(dirname(__file__), "test_data/350x150.jpg")


class TestWorkspace(object):
    def test_tmp(self):
        with tmp() as ofolder:
            assert os.path.exists(ofolder)
            assert os.path.isdir(ofolder)

        assert not os.path.exists(ofolder)

    def test_cache(self):
        @cache(u'./tmp/{0}-{1}-{2}')
        def _test(a, b, c, opath):
            with open(opath, 'w') as ofile:
                ofile.write('%s-%s-%s' % (a, b, c))

            return opath

        opath = _test('a', 'b', 'c')

        assert os.path.basename(opath) == 'a-b-c'
        assert os.path.exists(opath)
        assert open(opath).read() == 'a-b-c'

    def test_local(self):
        ofile = local('http://via.placeholder.com/350x150.jpg')

        assert os.path.basename(ofile).endswith('99f.jpe')

    def test_remote(self):
        url = remote(test_file)
        resp = requests.get(url)
        assert resp.status_code == 200



# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2017-11-09 07:23
from __future__ import unicode_literals

from django.db import migrations
import picklefield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('djworkspace', '0002_auto_20171109_0707'),
    ]

    operations = [
        migrations.AlterField(
            model_name='cache',
            name='results',
            field=picklefield.fields.PickledObjectField(editable=False, null=True),
        ),
    ]



from .models import Cache
from functools import wraps


def lrucache(key_str):
    def _lru_cache(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            key = key_str.format(*args, **kwargs)
            type = '{}.{}'.format(func.__module__, func.__name__)

            try:
                return Cache.objects.get(
                    key=key,
                    type=type
                ).results
            except Cache.DoesNotExist:
                result = func(*args, **kwargs)

                Cache.objects.create(
                    key=key,
                    type=type,
                    results=result
                )
                return result

        return wrapper

    return _lru_cache



# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2017-11-08 00:32
from __future__ import unicode_literals

from django.db import migrations, models
import picklefield.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Cache',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('type', models.CharField(max_length=40)),
                ('key', models.CharField(max_length=40)),
                ('results', picklefield.fields.PickledObjectField(editable=False)),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='cache',
            unique_together=set([('type', 'key')]),
        ),
    ]



import logging
import os
from contextlib import contextmanager

import workspace

try:
    from django.core.files import File
    from django.core.files.storage import DefaultStorage, FileSystemStorage
except ImportError:
    logging.warning("django_workspace require used in django framework")
    raise


def store(fieldfile, filepath):
    filename = os.path.basename(filepath)

    with open(filepath) as ifile:
        fieldfile.save(filename, File(ifile))


def remote(filepath, storage=DefaultStorage()):
    assert not isinstance(storage, FileSystemStorage), "Local FileSystem not support remote storage"
    assert os.path.exists(filepath), "File not exists"

    filename = os.path.basename(filepath)

    with open(filepath, 'rb') as ifile:
        return storage.save(filename, File(ifile))


def local(url_or_remote_file):
    if isinstance(url_or_remote_file, File):
        try:
            # NOTE: if it is a local file storage
            filepath = url_or_remote_file.path
            if os.path.exists(filepath):
                return filepath
        except Exception:
            pass

        if url_or_remote_file.storage.__class__.__name__.startswith('HashedFilename'):
            # NOTE: if it is a hashedfilename storage
            filename = os.path.basename(url_or_remote_file.name)
            return workspace._local(filename, url_or_remote_file.url)

        return workspace.local(url_or_remote_file.url)

    return workspace.local(url_or_remote_file)


@contextmanager
def temp_local(url_or_remote_file):
    filepath = local(url_or_remote_file)

    try:
        yield filepath
    finally:
        os.remove(filepath)



from os.path import dirname, exists, join

import pytest
from django.db import models

from djworkspace.store import local, remote, store
from fake.models import Fake

test_file = join(dirname(__file__), "test_data/350x150.jpg")



@pytest.mark.django_db
def test_store():
    test = Fake()
    store(test.field, test_file)

    assert open(test.field.path).read() == open(test_file).read()
    return test.field


@pytest.mark.django_db
def test_local():
    field = test_store()

    filepath = local(field)
    assert exists(filepath)


def test_remote():
    with pytest.raises(AssertionError):
        remote(test_file)
